/*
 * Copyright (c) 2007-2010 Sonatype, Inc. All rights reserved.
 *
 * "Sonatype" is a trademark of Sonatype, Inc.
 */

package org.sonatype.tycho.p2.bridge.plugin;

import java.io.File;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.sonatype.tycho.p2.bridge.client.P2DirectorFactory;
import org.sonatype.tycho.p2.bridge.client.P2ProfileRegistryFactory;
import org.sonatype.tycho.p2.bridge.facade.IUIdentity;
import org.sonatype.tycho.p2.bridge.facade.P2Director;
import org.sonatype.tycho.p2.bridge.facade.P2ProfileRegistry;
import org.sonatype.tycho.p2.eclipse.bridge.EclipseInstance;

/**
 * @goal list-available
 * @phase process-resources
 * @requiresDependencyResolution runtime
 */
public class ListAvailableMojo
    extends WithinEclipseRunningProjectBasedMojo
{

    /**
     * @parameter expression="${location}" default-value="${project.build.directory}/tycho-p2/p2-install-folder/p2"
     * @required
     */
    protected File location;

    /**
     * @parameter default-value="tycho-p2"
     * @required
     */
    private String profile;

    /**
     * @parameter
     * @required
     */
    private List<String> repositories;

    /**
     * @component
     */
    private P2DirectorFactory p2DirectorFactory;

    /**
     * @component
     */
    private P2ProfileRegistryFactory p2ProfileRegistryFactory;

    /**
     * @component
     */
    private Console console;

    @Override
    protected void doWithEclipse( final EclipseInstance eclipse )
    {
        console.printHeader( String.format( "Available versions for %s :", location.getAbsolutePath() ) );

        final P2Director p2Director = p2DirectorFactory.create( eclipse );
        final P2ProfileRegistry p2ProfileRegistry = p2ProfileRegistryFactory.create( eclipse );

        final PluginLogProxy logProxy = new PluginLogProxy( getLog() );

        final IUIdentity[] installedRoots = p2ProfileRegistry.getInstalledRoots( location, profile );

        final IUIdentity[] availableRoots =
            p2Director.getAvailableIUs( logProxy, toIdSet( installedRoots ), repositories );

        if ( toIdSet( availableRoots ).size() == 1 )
        {
            for ( final IUIdentity iu : availableRoots )
            {
                console.print( iu.getVersion() );
            }
        }
        else
        {
            for ( final IUIdentity iu : availableRoots )
            {
                console.print( iu.toString() );
            }
        }

        console.print();
    }

    private Set<String> toIdSet( final IUIdentity[] installedRoots )
    {
        final Set<String> roots = new HashSet<String>();
        for ( final IUIdentity iu : installedRoots )
        {
            roots.add( iu.getId() );
        }
        return roots;
    }
}
